function allevents = fgs_stats_withLL2(pathtosubjects,subj, fileroot, rtcutoff, binnum, binwidth, doebs)
% FUNCTION allevents = fgs_stats(pathtosubjects, prefix, subs,
% fileroot, rtcutoff, binnum, binwidth, doebs)
%
% Loads the events structures created by make_events, and plots
% accuracy and RT for the various modes by list-position probed.
% List length is hard-coded in this script. This version reverses
% the serial positions in the graphs
%
% INPUT ARGS:
%   pathtosubjects = '/home/ageller/mms/data'% the ABSOLUTE (full)
%                                            % path to the directory 
%                                            % containing the subjects' 
%                                            % directories
%   subj = {'FGS054','FGS055'}               % list of subjects
%   fileroot = 'beh/events.mat'              % how to get from subject's
%                                            % dir. to its events file
%   rtcutoff = 1000                          % trials with RTs above
%                                            % this value won't be included
%   binnum = 40                              % # of bins in RT-histogram
%   binwidth = 50                            % width in ms of in RT-hist.
%   doebs = 1                                % set this to 0 to prevent
%                                            % 95% confidence intervals
%                                            % from being drawn
%
% OUTPUT ARGS:
%   allevents- The events structure which contains all events
%   processed.

%default args:
if nargin<8
  doebs = 1;
  if nargin<7
    binwidth = 50;
    if nargin<6
      binnum = 40;
      if nargin<5
	rtcutoff = 1000;
      end
    end
  end
end
modelist = {'FACE' 'WORD' 'OBJECT' 'GRATING' 'LETTER' 'SPATIAL' };
allevents = loadSubjEvents(pathtosubjects,subj,fileroot);
if isfield(allevents,'ispractice')
  allevents = filterStruct(allevents,'~ispractice');
end
allcues = filterStruct(allevents, 'strcmp(type, ''CUE'')');
allcues = filterStruct(allcues,'rt>400');
do_plots(allcues, modelist, rtcutoff, binnum, binwidth, doebs);

function do_plots(events, modelist, rtcutoff, binnum, binwidth, doebs,doLM)
modes_to_do = which_modes_to_do(events, modelist);
colors_mat = prism(length(modelist));
subplot(2,2,1);
do_acc_plot(events, modelist, colors_mat, modes_to_do, rtcutoff,doebs);
publishfig;
subplot(2,2,2);
do_rt_plot(events, modelist, colors_mat, modes_to_do, rtcutoff,doebs);
publishfig;
subplot(2,2,[3 4]);
do_rt_hist(events, modelist, modes_to_do, binnum, binwidth);
publishfig;

function modes = which_modes_to_do(cueevents, modelist)
modes = [];
for i=1:length(modelist)
  query_str = ['strcmp(mode,''' modelist{i} ''')'];
  all_mode_cues = filterStruct(cueevents, query_str);
  if length(all_mode_cues)>0
    modes = [modes i];
  end
end

function do_acc_plot(events, modelist, colors_mat, modes_to_do,rtcutoff, doebs,doLM)
handle_vec = [];
for i=1:length(modes_to_do)
  handle_vec(end+1) = acc_plot_a_mode(events, modelist{modes_to_do(i)}, ...
				      colors_mat(modes_to_do(i),:), ...
				      rtcutoff, doebs);
end
legend(handle_vec,modelist{modes_to_do},0);
hold off;

function do_rt_plot(events, modelist, colors_mat, modes_to_do,rtcutoff, doebs,doLM)
for i=1:length(modes_to_do)
  rt_plot_a_mode(events, modelist{modes_to_do(i)}, ...
		 colors_mat(modes_to_do(i),:), rtcutoff, doebs);
end
hold off;

function do_rt_hist(events,modelist, modes_to_do, binnum,binwidth)
bins = 0:binwidth:binnum*binwidth;
bins(end+1) = inf; %as a catch-all final bin
colors_mat = prism(length(modelist));
counts = [];
cmap = [];
for i=1:length(modes_to_do)
    query_str = ['strcmp(mode,''' modelist{modes_to_do(i)} ''')'];
    mode_events = filterStruct(events,query_str);
    rts = getStructField(mode_events, 'rt');
    counts = [counts; histc(rts,bins)];
    cmap = [cmap; colors_mat(modes_to_do(i),:)]; 
    %set(h(2), 'Color', modecolor);
end
% set the colormap to be one row for each mode to be plotted
colormap(cmap);
h = bar(counts', 'group');
set(gca,'XTick',[1:5:binnum]);
set(gca,'XTickLabel',([1:5:binnum].*50-50));
xlabel('rt distribution (ms)');
legend(modelist{modes_to_do});


function hand = acc_plot_a_mode(events, modestr, colorvec, rtcutoff, doebs)
thisModeEvs = filterStruct(events,sprintf('strcmp(mode,''%s'')',modestr));
LLlist = unique(getStructField(thisModeEvs,'listlen'));
colorbinsize = 1/(length(LLlist)+2);
for ll = LLlist
    [all_list corr_list] = get_all_and_corr(events, modestr, rtcutoff,ll);
    ns = [];
    acc = zeros(1,ll+2);
    sdevs = zeros(1,ll+2); 
    for i=1:length(all_list)  
        ns(i) = length(all_list{i});
        numright = length(corr_list{i});
        if ns(i)==0
	  ns(i) = NaN; %to allow a point to be drawn
        end
        %vecInd = ll-i+2
	if i==1
	  vecInd = 1;
	else
	  vecInd = ll-i+3;
	end
        acc(vecInd) = numright/ns(i);
	if isnan(acc(vecInd))
	  sdevs(vecInd) = NaN;
	else
	  sdevs(vecInd) = std([ones(1,numright) zeros(1,ns(i)-numright)]);
	end
    end
    ns_mat = chunkify(ns,ll); %this part is a hack to
    acc_mat = chunkify(acc,ll); %prevent a line from being drawn
    sdevs_mat = chunkify(sdevs,ll);%from the "lure" point to the
                                   %others
    if doebs==1
        h = errorbar(acc_mat(1,:), 1.96*sdevs_mat(1,:)./(ns_mat(1,:).^.5), ...
	       'o-');
    else
        h = plot(acc_mat(1,:), 'o-');
    end
    curColor = ll*colorbinsize*colorvec;
    curColor(curColor>1) = 1;
    curColor(curColor<0) = 0;
    set_acc_attributes(h, curColor);
    hold on;
    if doebs==1
        h=errorbar(acc_mat(2,:), 1.96*sdevs_mat(2,:)./(ns_mat(2,:).^.5), ...
	     'o-');
        hand = h(1);
        set_acc_attributes(h, curColor);
    else
        hand = plot(acc_mat(2,:), 'o-');
        set_acc_attributes(hand, curColor);
    end
end

function hand = rt_plot_a_mode(events, modestr, colorvec, rtcutoff,doebs)
thisModeEvs = filterStruct(events,sprintf('strcmp(mode,''%s'')',modestr));
LLlist = unique(getStructField(thisModeEvs,'listlen'));
colorbinsize = 1/(length(LLlist)+2);
for ll = LLlist
    [all_list corr_list] = get_all_and_corr(events, modestr, rtcutoff,ll);
    ns = [];
    rts = zeros(1,ll+2);
    sdevs = zeros(1,ll+2);
    for i=1:length(corr_list)  
        ns(i) = length(corr_list{i});
        if ns(i)==0
            this_pos_rt = NaN;
        else
            this_pos_rt = getStructField(corr_list{i}, 'rt');
        end
	%vecInd = ll-i+2;
	if i==1
	  vecInd = 1; %lures are always on position 1
	else
	  vecInd = ll-i+3; %targets are on ll-serpos+3
	end
        rts(vecInd) = mean(this_pos_rt);
        sdevs(vecInd) = std(this_pos_rt);
    end
    ns_mat = chunkify(ns,ll); %this part is a hack to
    rt_mat = chunkify(rts,ll); %prevent a line from being drawn
    sdevs_mat = chunkify(sdevs,ll);%from the "lure" point to the others
    if doebs==1
        h=errorbar(rt_mat(1,:), 1.96*sdevs_mat(1,:)./(ns_mat(1,:).^.5),'o-');
    else
        h=plot(rt_mat(1,:));
    end
    % the color of the line will be graded according to the list length
    % where lighter versions reflect longer listlengths. We have to make
    % sure however that the color will never exceed 1
    curColor =ll*colorbinsize*colorvec;
    curColor(curColor > 1) = 1;
    curColor(curColor < 0) = 0; 
    set_rt_attributes(h, curColor );
    hold on;
    if doebs==1
        ns_mat(ns_mat==0) = NaN; 
        h=errorbar(rt_mat(2,:), 1.96*sdevs_mat(2,:)./(ns_mat(2,:).^.5),'o-');
	hand = h(1);
        set_rt_attributes(h, curColor);
    else
        hand=plot(rt_mat(2,:), 'o-');
        set_rt_attributes(hand, curColor);
    end
end

function [all_list, corr_list] = get_all_and_corr(cueevents, modestr, rtcutoff,LL)
%returns 2 lists of events for the requested mode, one with 
%all events at a particular position, one with just the events
%for the correct responses.
query_str = sprintf('strcmp(mode,''%s'') & rt<=%i & listlen==%i ', modestr, rtcutoff,LL);
all_mode_cues = filterStruct(cueevents, query_str);
all_targets = filterStruct(all_mode_cues, 'istarget');
all_list{1} = filterStruct(all_mode_cues, '~istarget');
for i=1:LL
  query_str = sprintf('probe_pos == %i', i);
  all_list{i+1} = filterStruct(all_targets, query_str);
end
corr_list{1} = filterStruct(all_list{1}, 'correct');
for i=1:LL
  corr_list{i+1} = filterStruct(all_list{i+1}, 'correct');
end

function chunks = chunkify(vec,LL)
NaNpart = repmat(NaN,1,LL);
beginchunk = [vec(1) NaNpart];
endchunk = [NaN vec(2:(LL+1))];
chunks = [beginchunk;endchunk];

function set_acc_attributes(handle, colorvec)
ylabel('Accuracy');
ylim([0 1.1]);
set_global_attributes(handle, colorvec);

function set_rt_attributes(handle, colorvec)
xlabel('position probed');
ylabel('RT (ms)');
set_global_attributes(handle, colorvec);

function set_global_attributes(handle, colorvec)
xlabel('position probed');
set(gca, 'XTick', [1 2 3 4 5]);
set(gca, 'XTickLabel', ['LURE '; 'POS-1';'POS-2'; 'POS-3'; 'POS-4']);
set(handle(1), 'color', colorvec);
set(handle(1), 'linewidth', 3);
if length(handle)==2
  set(handle(2), 'markerfacecolor', colorvec);
  set(handle(2), 'color', colorvec);
  set(handle(2), 'linewidth', 3);
end

