function makeconfig(block,filename,mode)
%function makeconfig(block,filename,mode)
% block = 1: practice block;    block=0: real block
% filename = the file to which the block should be appended
% mode = 1 for faces, mode=2 for gratings
%
% This function produces a practice or a real mode block for MMS, and it
% implements the following constraints/rules:
% 1) do not duplicate stimuli within lists
% 2) study items should not have appeared in the two preceding lists
% 3) target is randomly drawn from any of the current list items
% 4) list length is randomly determined, one of the numbers 2, 3 or 4
% ---- this one does not allow blocks of 30 ---5) an item is a lure only once per block
% 6) lures and targets appear with equal probability
% 7) a lure should by its definition not in the study list
% 8) each lure comes from 1, 2 or 3 lists back (with equal probabilities)

rand('seed',sum(100*clock)); %reset the random number generator
Nstimuli = 16;
%practiceLength = 16; %number of lists in practice blocks
practiceLength = 15;
realLength = 30; %number of lists in real blocks
enableOneLure = 0; % should be 1 only for short lists, i.e. shorter than 15 or so
% open file
fid = fopen(filename,'a');
%fprintf(fid,'#########\n');

  % lures have to be remembered since they are only allowed to appear once
  % in a block (16 is number of stimuli)
  rememberLure = zeros(1,Nstimuli);
% start the block in the file and prepare storing devices
if (block == 1)
  %fprintf(fid,'PRACTICE mode\n');
  fprintf(fid,sprintf('* %d %d %d\n',mode,block,1));
  trials = -ones(practiceLength,4);
  probes = -ones(practiceLength,1);
  Ntrials = practiceLength;
  tgts = randperm(Ntrials);
  % determine the list lengths for each trial
  %LLvec = ceil(3*rand(1,Ntrials)) + 1;
  LLvec = 3*ones(1,Ntrials);
else
    %fprintf(fid,'REAL mode\n');
    fprintf(fid,sprintf('* %d %d %d\n',mode,block,1));
    trials = -ones(realLength,4);
    probes = -ones(realLength,1);
    Ntrials = realLength;
    tgts = randperm(Ntrials);
    % determine the list lengths for each trial (randomly 2,3 or 4)
    %LLvec = ceil(3*rand(1,Ntrials)) + 1;
    LLvec = 3*ones(1,Ntrials);
end


for a = 1:Ntrials
  % determine whether the probe will be a target (if it's odd, it's a
  % target and if it's even, it's a lure)
  istarget = rem(tgts(a),2);
  
  % first create the study list, which should not have items from the
  % previous two trials in it
  thistrial = -ones(1,LLvec(a));
  for k = 1:LLvec(a)
    notOK = true;
    % draw a list of numbers (that do not duplicate within the list)
    while notOK
        drawNum = floor(Nstimuli*rand(1));
        % make sure the number is not already in the list
        while ~isempty(find(thistrial==drawNum))
            drawNum = floor(Nstimuli*rand(1));
        end
        
        thistrial(1,k) = drawNum;
        % check whether they have not occurred in earlier lists
        if a==1
            notOK = false;
        elseif a==2
            % check whether any of the stimuli in the generated list was part
            % of an earlier list (i.e. the first list in this case)
            if isempty(find(trials(1,:)==drawNum))
                notOK = false;
            end
        else %a>2
            % check whether the stimulus in the current list appears
            % in the trial before this or two trials before this
            if isempty(find(trials(a-2:a-1,:)==drawNum))
                notOK = false;
            end
        end
    end %while stimulus is not OK
  end % for move through list to find good stimuli in all positions

  % now we have found a good trial
  trials(a,1:LLvec(a)) = thistrial;
  
  if istarget
      % draw the target randomly from the list
      probeIndex = ceil(LLvec(a)*rand(1));
      probes(a) = thistrial(probeIndex);
      % fprintf(' target %i\n',thistrial(probeIndex));
  else %it's a lure
          lureOK = false;
          if a == 1
              tryLure = floor(Nstimuli*rand(1));
              while ~lureOK
                  %check whether lure has not been in this trial and
                  %whether it has not been lure in this block before
                if isempty(find(thistrial==tryLure)) & ~rememberLure(tryLure+1)
                  lureOK = true;
                else
                  tryLure = floor(Nstimuli*rand(1));
                end
              end
          elseif a == 2
              % get a position in the previous list
              probePos = ceil(LLvec(1)*rand(1));
              while ~lureOK
                  tryLure = trials(1,probePos);
                  %check whether it has not been lure before and not been
                  %probe in the previous trial
                  if ~rememberLure(tryLure+1) & (tryLure ~= probes(1))
                      lureOK = true;
                  else
                      probePos = ceil(LLvec(1)*rand(1));
                  end
              end
          elseif a == 3
              nback = ceil(2*rand(1)); %from how many lists back should the probe item come?
              probePos = ceil(LLvec(a-nback)*rand(1));
              while ~lureOK
                  tryLure = trials(a-nback,probePos);
                  %check whether it has not been lure before and not been
                  %probe in the previous 2 trials
                  if ~rememberLure(tryLure+1) & isempty(find(probes(1:2)==tryLure))
                      lureOK = true;
                  else
                      probePos = ceil(LLvec(a-nback)*rand(1));
                  end
              end  
          else % a>3
              nback = ceil(3*rand(1));
              % determine the probe position based on the list length for
              % the trial of interest
              probePos = ceil(LLvec(a-nback)*rand(1));
              while ~lureOK
                  tryLure = trials(a-nback,probePos);
                  %check whether it has not been lure before and not been
                  %probe in the previous 2 trials
                  if ~rememberLure(tryLure+1) & isempty(find(probes(a-2:a-1)==tryLure))
                      lureOK = true;
                  else
                      probePos = ceil(LLvec(a-nback)*rand(1));
                  end
              end %lure is now OK
          end %for a>3
        probes(a) = tryLure;
        % fprintf('lure %i\n',tryLure);
        if enableOneLure
            rememberLure(tryLure+1) = 1; %we insert it into position n+1 because even
        % though stimuli are numbered from 0-(Nstimuli-1), this vector only has indices
        % starting at 1 (and ending at Nstimuli)
        end
    end %it's a lure
  fprintf('%d ',a); %print the status of the algorithm
  switch LLvec(a)
      case 2
          fprintf(fid,'%i %i %i\n',thistrial(1),thistrial(2),probes(a));
      case 3
          fprintf(fid,'%i %i %i %i\n',thistrial(1),thistrial(2),thistrial(3),probes(a));
      case 4
          fprintf(fid,'%i %i %i %i %i\n',thistrial(1),thistrial(2),thistrial(3),thistrial(4),probes(a));
  end
end %for # of trials


